from autosklearn.experimental.askl2 import AutoSklearn2Classifier
import autosklearn.regression
import sklearn


def get_x_y(dataset):
    y = dataset["label"]
    X = dataset.drop("label", axis=1)
    return X, y


def automl(dataset, type):
    X, y = get_x_y(dataset)
    X_train, X_test, y_train, y_test = sklearn.model_selection.train_test_split(
        X, y, random_state=42
    )
    if type == "classification":
        automl = AutoSklearn2Classifier(
            time_left_for_this_task=120,
            per_run_time_limit=30,
            ensemble_nbest=5,
            tmp_folder="/tmp/autosklearn_classification_example_tmp",
        )
        automl.fit(X_train, y_train)
        predictions = automl.predict(X_test)
        return {
            "models": automl.show_models(),
            "score": sklearn.metrics.accuracy_score(y_test, predictions),
        }
    elif type == "regression":
        automl = autosklearn.regression.AutoSklearnRegressor(
            time_left_for_this_task=120,
            per_run_time_limit=30,
            ensemble_nbest=5,
            tmp_folder="/tmp/autosklearn_regression_example_tmp",
        )
        automl.fit(X_train, y_train)
        predictions = automl.predict(X_test)
        return {
            "models": automl.show_models(),
            "score": sklearn.metrics.r2_score(y_test, predictions),
        }



import requests


class Deployment:
    def __init__(self, deployment_id: str) -> None:
        self.deployment_id = deployment_id
        self.model_url = self.__get_deployment_details()

    def __get_deployment_details(self):
        try:
            response = requests.get(
                f"http://localhost:8000/deployments/sdk/{self.deployment_id}",
            )
            response = response.json()
        except:
            raise Exception("Unable to connect to the server")
        try:
            model_url = response["modelURL"]
        except:
            raise Exception("Invalid Deployment ID")
        return model_url

    def generate_code(self):
        with open("deployment.py", "w") as f:
            f.write("from pydantic import BaseModel")
            f.write("\nclass RequestBody(BaseModel):")
            f.write("\n\tdata: dict")
            f.write("\n")
            f.write("\ndef make_inference():")
            f.write("\n\t# TODO: Implement your inference logic here")


from sklearn.model_selection import train_test_split as sklearn_tts  # type: ignore
import pandas as pd
from ...error.decorators import function_error_handling
from ...connector import Dataset
import joblib
from ...util.data_types import get_dataframe_types

# @function_error_handling("train_test_split")
def train_test_split(**params):
    data = params["data"]
    train_dataset, test_dataset = sklearn_tts(
        data, test_size=params["parameters"]["test_size"], random_state=42
    )
    return train_dataset, test_dataset


# @function_error_handling("train_model")
def train_model(**params):
    data = params["data"]
    y = data[params["column"]]
    X = data.drop(params["column"], axis=1)
    data_dict  = get_dataframe_types(X)
    model = params["model"]
    Model = model.fit(X, y)
    filename = 'model.joblib'
    joblib.dump(model, filename)
    return Model, data_dict, filename



class ModelHandler:
    def __init__(self) -> None:
        pass

    def return_function(self, function_name: str, params: dict):
        models = {
            "Logistic Regression": self.__logistic_regression,
            "SVM (SVC)": self.__svm_svc,
            "Gaussian Naive Bayes": self.__gaussian_naive_bayes,
            "Multinomial Naive Bayes": self.__multinomial_naive_bayes,
            "Stochastic Gradient Descent Classifier": self.__st_gradient_descent_classifier,
            "KNN": self.__knn,
            "Decision Tree Classifier": self.__decision_tree,
            "Random Forest Classifier": self.__random_forest,
            "Gradient Boosting Classifier": self.__gradient_boosting,
            "LGBM Classifier": self.__lgbm,
            "XGBoost Classifier": self.__xgboost,
            "Linear Regression": self.__linear_regression,
            "LGBM Regressor": self.__lgbm_regressor,
            "XGBoost Regressor": self.__xgboost_regressor,
            "CatBoost Regressor": self.__catboost_regressor,
            "Stochastic Gradient Descent Regression": self.__st_gradient_descent_regressor,
            "Kernel Ridge Regression": self.__kernel_ridge,
            "Elastic Net Regression": self.__elastic_net,
            "Bayesian Ridge Regression": self.__bayesian_ridge,
            "Gradient Boosting Regression": self.__gradient_boosting_reg,
            "SVM (SVR)": self.__svm_svr,
            "Mean Shift": self.__mean_shift,
            "KMeans": self.__kmeans,
            "Agglomerative Clustering": self.__agglomerative,
            "BIRCH": self.__birch,
            "Spectral Clustering": self.__spectral,
            "Affinity Propagation": self.__affinity_propogation,
            "OPTICS": self.__optics,
            "DBSCAN": self.__dbscan,
        }
        return models[function_name](params)

    def __logistic_regression(self, params):
        parameters = {
            "penalty": params["penalty"],
            "fit_intercept": params["fit_intercept"],
            "random_state": params["random_state"],
            "solver": params["solver"],
            "max_iter": params["max_iter"],
            "multi_class": params["multi_class"],
            "tol": params["tol"],
        }
        return f"model = huble.sklearn.logistic_regression({parameters})"

    def __svm_svc(self, params):
        parameters = {
            "C": params["C"],
            "kernel": params["kernel"],
            "probability": params["probability"],
            "random_state": params["random_state"],
            "max_iter": params["max_iter"],
            "decision_function_shape": params["decision_function_shape"],
            "tol": params["tol"],
        }
        return f"model = huble.sklearn.svm_svc(parameters={parameters})"

    def __gaussian_naive_bayes(self, params):
        parameters = {
            "priors": params["priors"],
            "var_smoothing": params["var_smoothing"],
        }
        return f"model = huble.sklearn.gaussian_naive_bayes(parameters={parameters})"

    def __multinomial_naive_bayes(self, params):
        parameters = {
            "class_prior": params["class_prior"],
            "alpha": params["alpha"],
            "fit_prior": params["fit_prior"],
        }
        return f"model = huble.sklearn.multinomial_naive_bayes(parameters={parameters})"

    def __st_gradient_descent_classifier(self, params):
        parameters = {
            "loss": params["loss"],
            "penalty": params["penalty"],
            "fit_intercept": params["fit_intercept"],
            "alpha": params["alpha"],
            "max_iter": params["max_iter"],
            "tol": params["tol"],
            "random_state": params["random_state"],
            "shuffle": params["shuffle"],
            "learning_rate": params["learning_rate"],
            "initial_learning_rate": params["initial_learning_rate"],
            "early_stopping": params["early_stopping"],
            "validation_fraction": params["validation_fraction"],
        }
        return f"model = huble.sklearn.st_gradient_descent_classifier(parameters={parameters})"

    def __knn(self, params):
        parameters = {
            "n_neighbors": params["n_neighbors"],
            "weights": params["weights"],
            "algorithm": params["algorithm"],
            "metric": params["metric"],
        }
        return f"model = huble.sklearn.knn(parameters={parameters})"

    def __decision_tree(self, params):
        parameters = {
            "criterion": params["criterion"],
            "splitter": params["splitter"],
            "max_depth": params["max_depth"],
            "max_leaf_nodes": params["max_leaf_nodes"],
            "random_state": params["random_state"],
        }
        return f"model = huble.sklearn.decision_tree(parameters={parameters})"

    def __random_forest(self, params):
        for param in params:
            if params[param] == "None":
                params[param] = None
        parameters = {
            "criterion": params["criterion"],
            "n_estimators": params["n_estimators"],
            "max_depth": params["max_depth"],
            "max_leaf_nodes": params["max_leaf_nodes"],
            "random_state": params["random_state"],
        }
        return f"model = huble.sklearn.random_forest(parameters={parameters})"

    def __gradient_boosting(self, params):
        parameters = {
            "criterion": params["criterion"],
            "n_estimators": params["n_estimators"],
            "max_depth": params["max_depth"],
            "max_leaf_nodes": params["max_leaf_nodes"],
            "random_state": params["random_state"],
            "loss": params["loss"],
            "learning_rate": params["learning_rate"],
            "subsample": params["subsample"],
            "tol": params["tol"],
        }
        return f"model = huble.sklearn.gradient_boosting(parameters={parameters})"

    def __lgbm(self, params):
        parameters = {
            "boosting_type": params["boosting_type"],
            "num_leaves": params["num_leaves"],
            "n_estimators": params["n_estimators"],
            "max_depth": params["max_depth"],
            "random_state": params["random_state"],
            "learning_rate": params["learning_rate"],
        }
        return f"model = huble.sklearn.lgbm(parameters={parameters})"

    def __xgboost(self, params):
        parameters = {
            "n_estimators": params["n_estimators"],
            "max_depth": params["max_depth"],
            "random_state": params["random_state"],
            "learning_rate": params["learning_rate"],
        }
        return f"model = huble.sklearn.xgboost(parameters={parameters})"

    # Regression

    def __linear_regression(self, params):
        parameters = {
            "fit_intercept": params["fit_intercept"],
            "normalize": params["normalize"],
            "copy_X": params["copy_X"],
            "positive": params["positive"],
        }
        return f"model = huble.sklearn.linear_regression(parameters={parameters})"

    def __lgbm_regressor(self, params):
        parameters = {
            "boosting_type": params["boosting_type"],
            "num_leaves": params["num_leaves"],
            "n_estimators": params["n_estimators"],
            "max_depth": params["max_depth"],
            "random_state": params["random_state"],
            "learning_rate": params["learning_rate"],
        }
        return f"model = huble.sklearn.lgbm_reg(parameters={parameters})"

    def __xgboost_regressor(self, params):
        parameters = {
            "n_estimators": params["n_estimators"],
            "max_depth": params["max_depth"],
            "random_state": params["random_state"],
            "learning_rate": params["learning_rate"],
        }
        return f"model = huble.sklearn.xgboost_reg(parameters={parameters})"

    def __catboost_regressor(self, params):
        parameters = {
            "iterations": params["iterations"],
            "learning_rate": params["learning_rate"],
        }
        return f"model = huble.sklearn.catboost_reg(parameters={parameters})"

    def __st_gradient_descent_regressor(self, params):
        parameters = {
            "loss": params["loss"],
            "penalty": params["penalty"],
            "fit_intercept": params["fit_intercept"],
            "alpha": params["alpha"],
            "max_iter": params["max_iter"],
            "tol": params["tol"],
            "random_state": params["random_state"],
            "shuffle": params["shuffle"],
            "learning_rate": params["learning_rate"],
            "early_stopping": params["early_stopping"],
        }
        return f"model = huble.sklearn.st_gradient_descent_reg(parameters={parameters})"

    def __kernel_ridge(self, params):
        parameters = {
            "alpha": params["alpha"],
        }
        return f"model = huble.sklearn.kernel_ridge_reg(parameters={parameters})"

    def __elastic_net(self, params):
        parameters = {
            "fit_intercept": params["fit_intercept"],
            "normalize": params["normalize"],
            "copy_X": params["copy_X"],
            "positive": params["positive"],
            "alpha": params["alpha"],
            "l1_ratio": params["l1_ratio"],
            "selection": params["selection"],
        }
        return f"model = huble.sklearn.elastic_net_reg(parameters={parameters})"

    def __bayesian_ridge(self, params):
        parameters = {
            "n_iter": params["n_iter"],
        }
        return f"model = huble.sklearn.bayesian_ridge_reg(parameters={parameters})"

    def __gradient_boosting_reg(self, params):
        parameters = {
            "criterion": params["criterion"],
            "n_estimators": params["n_estimators"],
            "max_depth": params["max_depth"],
            "max_leaf_nodes": params["max_leaf_nodes"],
            "random_state": params["random_state"],
            "loss": params["loss"],
            "learning_rate": params["learning_rate"],
            "subsample": params["subsample"],
            "tol": params["tol"],
        }
        return f"model = huble.sklearn.gradient_boosting_reg(parameters={parameters})"

    def __svm_svr(self, params):
        parameters = {
            "C": params["C"],
            "kernel": params["kernel"],
            "max_iter": params["max_iter"],
            "tol": params["tol"],
        }
        return f"model = huble.sklearn.svm_svr(parameters={parameters})"

    # Clustering

    def __mean_shift(self, params):
        parameters = {
            "bandwidth": params["bandwidth"],
            "bin_seeding": params["bin_seeding"],
            "min_bin_freq": params["min_bin_freq"],
            "cluster_all": params["cluster_all"],
            "max_iter": params["max_iter"],
        }
        return f"model = huble.sklearn.mean_shift(parameters={parameters})"

    def __kmeans(self, params):
        parameters = {
            "n_clusters": params["n_clusters"],
            "init": params["init"],
            "copy_X": params["copy_X"],
            "tol": params["tol"],
            "max_iter": params["max_iter"],
            "algorithm": params["algorithm"],
            "random_state": params["random_state"],
        }
        return f"model = huble.sklearn.kmeans(parameters={parameters})"

    def __agglomerative(self, params):
        parameters = {
            "n_clusters": params["n_clusters"],
            "linkage": params["linkage"],
        }
        return f"model = huble.sklearn.agglomerative(parameters={parameters})"

    def __birch(self, params):
        parameters = {
            "threshold": params["threshold"],
            "branching_factor": params["branching_factor"],
            "n_clusters": params["n_clusters"],
        }
        return f"model = huble.sklearn.birch(parameters={parameters})"

    def __spectral(self, params):
        parameters = {
            "n_clusters": params["n_clusters"],
            "eigen_solver": params["eigen_solver"],
            "n_init": params["n_init"],
            "assign_labels": params["assign_labels"],
        }
        return f"model = huble.sklearn.spectral(parameters={parameters})"

    def __affinity_propogation(self, params):
        parameters = {
            "damping": params["damping"],
            "max_iter": params["max_iter"],
            "convergence_iter": params["convergence_iter"],
            "affinity": params["affinity"],
        }
        return f"model = huble.sklearn.affinity_propogation(parameters={parameters})"

    def __optics(self, params):
        parameters = {
            "min_samples": params["min_samples"],
            "algorithm": params["algorithm"],
        }
        return f"model = huble.sklearn.optics(parameters={parameters})"

    def __dbscan(self, params):
        parameters = {
            "eps": params["eps"],
            "min_samples": params["min_samples"],
            "algorithm": params["algorithm"],
        }
        return f"model = huble.sklearn.dbscan(parameters={parameters})"



from sklearn import metrics


def regression_metrics(y_true, y_pred):
    max_error = metrics.max_error(y_true, y_pred)
    mean_absolute_error = metrics.mean_absolute_error(y_true, y_pred)
    mean_squared_error = metrics.mean_squared_error(y_true, y_pred)
    mean_squared_log_error = metrics.mean_squared_log_error(y_true, y_pred)
    median_absolute_error = metrics.median_absolute_error(y_true, y_pred)
    r2_score = metrics.r2_score(y_true, y_pred)
    mean_poisson_deviance = metrics.mean_poisson_deviance(y_true, y_pred)
    mean_gamma_deviance = metrics.mean_gamma_deviance(y_true, y_pred)
    mean_tweedie_deviance = metrics.mean_tweedie_deviance(y_true, y_pred)
    # create dictionary of metrics
    metrics_dict = {
        "max_error": max_error,
        "mean_absolute_error": mean_absolute_error,
        "mean_squared_error": mean_squared_error,
        "mean_squared_log_error": mean_squared_log_error,
        "median_absolute_error": median_absolute_error,
        "r2_score": r2_score,
        "mean_poisson_deviance": mean_poisson_deviance,
        "mean_gamma_deviance": mean_gamma_deviance,
        "mean_tweedie_deviance": mean_tweedie_deviance,
    }
    return metrics_dict


def classification_metrics(y_true, y_pred):
    accuracy_score = metrics.accuracy_score(y_true, y_pred)
    balanced_accuracy_score = metrics.balanced_accuracy_score(y_true, y_pred)
    cohen_kappa_score = metrics.cohen_kappa_score(y_true, y_pred)
    f1_score = metrics.f1_score(y_true, y_pred)
    fbeta_score = metrics.fbeta_score(y_true, y_pred, beta=0.5)
    hamming_loss = metrics.hamming_loss(y_true, y_pred)
    jaccard_score = metrics.jaccard_score(y_true, y_pred)
    log_loss = metrics.log_loss(y_true, y_pred)
    matthews_corrcoef = metrics.matthews_corrcoef(y_true, y_pred)
    precision_score = metrics.precision_score(y_true, y_pred)
    recall_score = metrics.recall_score(y_true, y_pred)
    zero_one_loss = metrics.zero_one_loss(y_true, y_pred)
    # create dictionary of metrics
    metrics_dict = {
        "accuracy_score": accuracy_score,
        "balanced_accuracy_score": balanced_accuracy_score,
        "cohen_kappa_score": cohen_kappa_score,
        "f1_score": f1_score,
        "fbeta_score": fbeta_score,
        "hamming_loss": hamming_loss,
        "jaccard_score": jaccard_score,
        "log_loss": log_loss,
        "matthews_corrcoef": matthews_corrcoef,
        "precision_score": precision_score,
        "recall_score": recall_score,
        "zero_one_loss": zero_one_loss,
    }
    return metrics_dict





def log_metrics(y_true, y_pred, task):
    if task == "regression":
        metrics_dict = regression_metrics(y_true, y_pred)
        return metrics_dict
    elif task == "classification":
        metrics_dict = classification_metrics(y_true, y_pred)
        return metrics_dict



import huble
from huble import Dataset
def run_experiment(experiment):
	model = huble.sklearn.random_forest(parameters={'criterion': 'gini', 'n_estimators': 70, 'max_depth': 2, 'max_leaf_nodes': 10, 'random_state': None})
	data = Dataset('https://ipfs.filebase.io/ipfs/QmRspeqXi9J2PVTmXYwMaBif9dYWVkNhM8EFomUAfajnT1').dataframe
	data = huble.sklearn.drop_duplicates(data=data,parameters={'subset': [], 'keep': 'first', 'inplace': False, 'ignore_index': False})
	data = huble.sklearn.remove_mismatch_data(data=data, parameters={'exceptions': []})
	data = huble.sklearn.remove_outliers(data=data,columns=['Fare'])
	data = huble.sklearn.drop_rows_columns(data=data,parameters={'labels': ['PassengerId', 'Pclass', 'Name'], 'axis': 1, 'inplace': False, 'errors': 'raise'})
	data = huble.sklearn.clean_data(data=data)
	training_dataset, test_dataset = huble.sklearn.train_test_split(data=data,parameters={'test_size': 0.8})
	Model, input_format, filename = huble.sklearn.train_model(data=training_dataset, model=model, column='Survived')
	metrics = huble.sklearn.evaluate_model(model=Model, test_dataset=test_dataset, target_column= 'Survived', task_type='classification' )
	experiment.upload_metrics(metrics,input_format)
	experiment.upload_model(filename)


import requests
from boto3.session import Session


class Experiment:
    def __init__(self, experiment_id: str) -> None:
        self.experiment_id = experiment_id
        self.graph, self.task_type, self.target_column = self.__get_experiment_details(experiment_id=experiment_id)

    def __get_experiment_details(self, experiment_id: str):
        try:
            response = requests.get(
                f"http://localhost:8000/experiments/{experiment_id}",
            )
            response = response.json()
        except:
            raise Exception("Unable to connect to the server")
        try:
            graph = response["pipelineJSON"]
            project = response["project"]
            target_column = project["targetColumn"]
            task_type = project["taskType"]
        except:
            raise Exception("Invalid Experiment ID")
        return graph, task_type, target_column

    def upload_metrics(self, metrics, input_format):
        requests.put(
            f"http://localhost:8000/experiments/results/{self.experiment_id}",
            data={"metrics": metrics, "input_format": input_format},
        )

    def upload_model(self, file_name):
        session = Session()
        client = session.client(
            "s3",
            region_name="us-east-1",
            endpoint_url="https://s3.filebase.com",
            aws_access_key_id="709B6E02413B7282AC93",
            aws_secret_access_key="EyByFcqsBwTe4bWq6PwnoUJ9e2BqzShztJ48efVH",
        )

        bucket_name = "testsesaetsatast"
        with open(file_name, "rb") as data:
            client.upload_fileobj(data, bucket_name, file_name)
